<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Block\Click;

use Flat101\Base\Service\GetCurrentProductService as CurrentProduct;
use Flat101\Base\Service\GetCurrentCategoryService as CurrentCategory;
use Flat101\EnhancedEcommerce\Block\DataLayer;
use Flat101\EnhancedEcommerce\Model\DataLayer\Impressions\CatalogImpressionView as CatalogImpressionViewUa;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Impressions\CatalogImpressionView as CatalogImpressionViewGa4;
use Flat101\EnhancedEcommerce\Model\DataLayer\Impressions\ProductRelatedView as ProductRelatedViewUa;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Impressions\ProductRelatedView as ProductRelatedViewGa4;
use Flat101\EnhancedEcommerce\Model\DataLayer\Impressions\SearchImpressionView as SearchImpressionViewUa;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Impressions\SearchImpressionView as SearchImpressionViewGa4;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Impressions\CheckoutImpressionView as CheckoutImpressionViewGa4;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Impressions\CheckoutImpressionView as CheckoutImpressionViewUa;
use Flat101\EnhancedEcommerce\Api\DataLayerListInterface;
use Flat101\EnhancedEcommerce\Factory\DataLayerFactory;
use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Framework\View\Element\Template;
use Magento\Framework\App\Response\RedirectInterface;
use Magento\Framework\Event\Manager;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Framework\UrlInterface;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;

class Data extends DataLayer
{
    /** @var CatalogImpressionViewUa */
    protected $catalogImpressionView;

    /** @var CatalogImpressionViewGa4 */
    protected $catalogImpressionViewGa4;

    /** @var ProductRelatedViewUa */
    protected $relatedImpressionView;

    /** @var ProductRelatedViewGa4 */
    protected $relatedImpressionViewGa4;

    /** @var SearchImpressionViewUa */
    protected $searchImpressionView;

    /** @var SearchImpressionViewGa4 */
    protected $searchImpressionViewGa4;

    /** @var CurrentCategory */
    protected $currentCategory;

    /** @var CollectionFactory */
    protected $collectionFactory;

    /**
     * @var CheckoutImpressionViewGa4
     */
    private  $checkoutImpressionViewGa4;
    /**
     * @var CheckoutImpressionViewUa
     */
    private  $checkoutImpressionViewUa;

    public function __construct(
        CurrentProduct $currentProduct,
        CurrentCategory $currentCategory,
        CatalogImpressionViewUa $catalogImpressionView,
        CatalogImpressionViewGa4 $catalogImpressionViewGa4,
        ProductRelatedViewUa $relatedImpressionView,
        ProductRelatedViewGa4 $relatedImpressionViewGa4,
        SearchImpressionViewUa $searchImpressionView,
        SearchImpressionViewGa4 $searchImpressionViewGa4,
        CheckoutImpressionViewGa4 $checkoutImpressionViewGa4,
        CheckoutImpressionViewUa $checkoutImpressionViewUa,
        Template\Context $context,
        DataLayerListInterface $dataLayerList,
        DataLayerFactory $dataLayerFactory,
        SerializerInterface $serializer,
        UrlInterface $url,
        Config $config,
        RedirectInterface $redirect,
        Manager $eventManage,
        CollectionFactory $collectionFactory,
        array $data = []
    ) {
        parent::__construct($currentProduct, $context, $dataLayerList, $dataLayerFactory, $serializer, $url, $config, $redirect, $eventManage, $data);
        $this->catalogImpressionView = $catalogImpressionView;
        $this->catalogImpressionViewGa4 = $catalogImpressionViewGa4;
        $this->relatedImpressionView = $relatedImpressionView;
        $this->relatedImpressionViewGa4 = $relatedImpressionViewGa4;
        $this->searchImpressionView = $searchImpressionView;
        $this->searchImpressionViewGa4 = $searchImpressionViewGa4;
        $this->checkoutImpressionViewGa4 = $checkoutImpressionViewGa4;
        $this->checkoutImpressionViewUa = $checkoutImpressionViewUa;
        $this->serializer = $serializer;
        $this->config = $config;
        $this->currentCategory = $currentCategory;
        $this->collectionFactory = $collectionFactory;
    }

    public function getJsonClickData(): ?string
    {
        $impressionDataType = null;

        $currentFullAction = $this->getRequest()->getFullActionName();
        $plpPage = ['catalog_category_view','ambrand_index_index'];
        $pdpPage = ['catalog_product_view'];
        $searchPage = ['catalogsearch_result_index'];
        $homePage = ['cms_index_index'];

        if (in_array($currentFullAction, $homePage)) {
            $impressionDataType['GA4'] = $this->catalogImpressionViewGa4->getLayer();

            if (!$this->config->getIsDisabledUaDatalayer()) {
                $impressionDataType['UA'] = $this->catalogImpressionView->getLayer();
            }
        }

        if (in_array($currentFullAction, $plpPage)) {
            $impressionDataType['GA4'] = $this->catalogImpressionViewGa4->getLayer();

            if (!$this->config->getIsDisabledUaDatalayer()) {
                $impressionDataType['UA'] = $this->catalogImpressionView->getLayer();
            }
        }

        if (in_array($currentFullAction, $pdpPage)) {
            // TODO: GA4
            $impressionDataType['GA4'] = $this->relatedImpressionViewGa4->getImpressionsData();

            if (!$this->config->getIsDisabledUaDatalayer()) {
                $impressionDataType['UA'] = $this->relatedImpressionView->getImpressionsData();
            }
        }

        if (in_array($currentFullAction, $searchPage)) {
            // TODO: GA4
            $impressionDataType['GA4'] = $this->searchImpressionViewGa4->getLayer();
            if (!$this->config->getIsDisabledUaDatalayer()) {
                $impressionDataType['UA'] = $this->searchImpressionView->getLayer();
            }
        }

        if (is_null($impressionDataType))
            return $this->serializer->serialize('');

        $response = [];

        // TODO: Test
        foreach ($impressionDataType as $type => $impressionData) {
            if (isset($impressionData)) {
                foreach ($impressionData as $impression) {
                    if ($type === 'UA') {
                        /** @var \Flat101\EnhancedEcommerce\Model\DataLayer\Dto\Impression\ImpressionProduct $impression */
                        $prodId = $impression->id;
                        if ($this->config->getProductIdentifier() === 'sku') {
                            $prodId = $this->getProdIdBySku($impression->id);
                        }

                        $response[$type][$prodId] = [
                            'list'     => $impression->list,
                            'category' => $impression->category,
                            'position' => $impression->position,
                        ];
                    } elseif ($type === 'GA4') {
                        /** @var \Flat101\EnhancedEcommerce\Model\DataLayerGa4\Dto\Impression\ImpressionProduct $impression */
                        $prodId = $impression->item_id;
                        if ($this->config->getProductIdentifier() === 'sku') {
                            $prodId
                                = $this->getProdIdBySku($impression->item_id);
                        }

                        $response[$type][$prodId] = [
                            'list'     => $impression->item_list_name,
                            'position' => $impression->index,
                        ];

                        if (isset($impression->item_category)) {
                            $response[$type][$prodId]['item_category'] = $impression->item_category;
                        }
                        if (isset($impression->item_category2)) {
                            $response[$type][$prodId]['item_category2'] = $impression->item_category2;
                        }
                        if (isset($impression->item_category3)) {
                            $response[$type][$prodId]['item_category3'] = $impression->item_category3;
                        }
                        if (isset($impression->item_category4)) {
                            $response[$type][$prodId]['item_category4'] = $impression->item_category4;
                        }
                        if (isset($impression->item_category5)) {
                            $response[$type][$prodId]['item_category5'] = $impression->item_category5;
                        }
                    }
                }
            }
        }
        return $this->serializer->serialize($response);
    }

    public function getAjaxUrl(): string
    {
        return $this->getUrl('flat101_enhancedecommerce/click/view');
    }

    protected function getProdIdBySku($sku): string
    {
        $collection = $this->collectionFactory->create()
            ->addAttributeToSelect('entity_id')
            ->addAttributeToFilter('sku', $sku);

        return $collection->getFirstItem()->getId();
    }
}
