<?php declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Helper;

use \Magento\Framework\App\Helper\AbstractHelper;

class AnalyticsConnector extends AbstractHelper
{
    const GA_PROPERTY_ID = "flat101_analytics/general/gtm_code";

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var \Magento\Framework\HTTP\Client\Curl
     */
    protected $_curl;

    /**
     * @var \Magento\Framework\HTTP\Header
     */
    protected $_http;

    /**
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param \Magento\Framework\HTTP\Header $http
     */
    public function __construct(
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\HTTP\Header $http
    ) {
        $this->_storeManager = $storeManager;
        $this->_scopeConfig = $scopeConfig;
        $this->_curl = $curl;
        $this->_http = $http;
    }

    public function sendOrderToGoogle($order)
    {
        $data = [];
        $url = 'https://www.google-analytics.com/collect';
        $debugUrl = "https://www.google-analytics.com/debug/collect"; // Change url for testing the API

        if ($order->getId()) {
            $store = $this->_storeManager->getStore($order->getStoreId());

            $gaId = $this->_scopeConfig->getValue(self::GA_PROPERTY_ID,
                \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
                $store);

            $affiliation = 'admin'; // To recognize that order is placed in admin
            if ($gaId) {
                $data['v'] = 1;
                $data['tid'] = $gaId;
                $data['cid'] = $order->getCustomerId();
                $data['t'] = 'transaction';
                $data['ta'] = $affiliation;
                $data['ti'] = $order->getIncrementId();
                $data['tr'] = $order->getGrandTotal();
                $data['ts'] = $order->getShippingAmount();
                $data['tt'] = $order->getTaxAmount();
                $data['cu'] = 'USD';

                if ($this->sendData($data, $url)) {
// If order is sent successfully send order items
                    $data = [];
                    $data['v'] = 1;
                    $data['tid'] = $gaId;
                    $data['cid'] = $order->getCustomerId();
                    $data['t'] = 'item';
                    $data['ti'] = $order->getIncrementId();
                    foreach ($order->getAllVisibleItems() as $item) {
                        $data['in'] = $item->getName();
                        $data['ip'] = $item->getPrice();
                        $data['iq'] = $item->getQtyOrdered();
                        $data['ic'] = $item->getSku();
                        $data['cu'] = 'USD';
                        $this->sendData($data, $url);
                    }
                }
            }
        }
    }

    private function sendData($data, $url)
    {
        $url = $url . "?" . http_build_query($data);
        $userAgent = $this->_http->getHttpUserAgent();
        if (!$userAgent) { // User agent is a required option
            $userAgent = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/20080311
Firefox/2.0.0.13';
        }

        $curlOptions = array(
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_TIMEOUT => 10,
            CURLOPT_USERAGENT => $userAgent
        );
        try {
            $this->_curl->setOptions($curlOptions);
            $this->_curl->get($url);
            $response = $this->_curl->getBody();
            if ($this->_curl->getStatus() != 200) {
                return false;
            }
        } catch (Exception $e) {
            return false;
        }
        return true;
    }
}
