<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayer;

use Flat101\EnhancedEcommerce\Api\Data\DataLayerInterface;
use Flat101\EnhancedEcommerce\Model\Session as SessionManager;
use Magento\Checkout\Model\Session;

class CartView implements DataLayerInterface
{
    public const CODE = 'cart-view';

    private const STEP = 1;
    private const STEP_LABEL = '';
    private const OPTION = 'cart';

    /**
     * @var Session
     */
    private $session;

    /**
     * @var SessionManager
     */
    private $sessionManager;

    /**
     * @var Generator\CheckoutStep
     */
    private $checkoutStep;

    /**
     * @var Generator\Product
     */
    private $productGenerator;

    public function __construct(
        Session $session,
        SessionManager $sessionManager,
        Generator\CheckoutStep $checkoutStep,
        Generator\Product $productGenerator
    ) {
        $this->session = $session;
        $this->sessionManager = $sessionManager;
        $this->checkoutStep = $checkoutStep;
        $this->productGenerator = $productGenerator;
    }

    public function getCode(): string
    {
        return static::CODE;
    }

    public function getLayer(): Dto\Ecommerce
    {
        // TODO: Add config to select if cart is first step from Admin Panel.
        //  Right now, this is not used.
        return $this->checkoutStep->onCheckoutStep(self::STEP, '', self::STEP_LABEL);
    }

    public function getCartLayer(): array
    {
        return $this->sessionManager->getAddProductToCart(true);
    }

    public function getRemoveCartLayer(): array
    {
        return $this->sessionManager->getRemovedProductFromCart(true);
    }
}
